==> mod_accel-1.0.17.tar.gz: mod_accel-1.0.17/mod_accel/ap_other_child.c <==
/*
 * Copyright (C) 2001 Igor Sysoev.
 */

#include "ap_other_child.h"

API_EXPORT(void) ap_start_other_child(server_rec *s, pool *p,
                          int (*func)(void *data), void *data)
{
    /* !one_process */

    other_child_rec *child = ap_palloc(p, sizeof(other_child_rec));
    child->pid = -1;
    child->server = s;
    child->func = func;
    child->data = data;
    ap_register_cleanup(p, child, ap_other_child_cleanup, ap_null_cleanup);

    if (ap_spawn_other_child(child, func, data) == -1) {
        ap_log_error(APLOG_MARK, APLOG_ERR, s, "can't spawn other child");
        ap_kill_cleanup(p, child, ap_other_child_cleanup);
    }

#ifdef MOD_ACCEL_DEBUG
    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, s,
                 "accel: fork %d, %d", getpid(), child->pid);
#endif

}

int ap_spawn_other_child(other_child_rec *child,
                         int (*func)(void *data), void *data)
{
    int pid;

    ap_block_alarms();
    pid = fork();
    switch (pid) {
    case -1:     /* error */
        break;

    case 0:      /* child */
        RAISE_SIGSTOP(SPAWN_CHILD);
        signal(SIGHUP, SIG_IGN);
        signal(SIGTERM, SIG_DFL);
        signal(SIGUSR1, SIG_IGN);

#ifdef MOD_ACCEL_DEBUG
    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, child->server,
                 "accel: child %d", pid);
#endif

        if ((*func)(data) == -1)
             exit(1);

        exit(0);

    default:     /* parent */
        child->pid = pid;
        ap_register_other_child(pid, ap_other_child_maintenance, child, -1);
    }
    ap_unblock_alarms();
    return pid;
}

static void ap_other_child_maintenance(int reason,
                                       void *data, ap_wait_t status)
{
    other_child_rec *child = data;

#ifdef MOD_ACCEL_DEBUG
    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, child->server,
                 "accel: maintenance %d, %d, %d, %04x",
                 getpid(), child->pid, reason, status);
#endif

    switch (reason) {
    case OC_REASON_DEATH:
    case OC_REASON_LOST:
        if (child->pid != -1)
            kill(child->pid, SIGKILL);
        child->pid = -1;
        ap_unregister_other_child(child);

        if (status == 0x100)
            break;

        if (ap_spawn_other_child(child, child->func, child->data) == -1) {
            ap_log_error(APLOG_MARK, APLOG_ERR, child->server,
                         "can't spawn other child");
        }

        break;

    case OC_REASON_RESTART:
        if (child->pid != -1)
            kill(child->pid, SIGKILL);
        child->pid = -1;
    }
}

static void ap_other_child_cleanup(void *data)
{
    other_child_rec *child = data;

    if (child->pid != -1)
        kill(child->pid, SIGKILL);

    ap_unregister_other_child(child);
}

==> mod_accel-1.0.20.tar.gz: mod_accel-1.0.20/mod_accel/ap_other_child.c <==
/*
 * Copyright (C) 2001 Igor Sysoev.
 */

#include "ap_other_child.h"

static int ap_spawn_other_child(other_child_rec *child,
                                int (*func)(void *data), void *data);
static void ap_other_child_maintenance(int reason,
                                       void *data, ap_wait_t status);
static void ap_other_child_cleanup(void *data);


API_EXPORT(void) ap_start_other_child(server_rec *s, pool *p,
                          int (*func)(void *data), void *data)
{
    /* !one_process */

    other_child_rec *child = ap_palloc(p, sizeof(other_child_rec));
    child->pid = -1;
    child->server = s;
    child->func = func;
    child->data = data;
    ap_register_cleanup(p, child, ap_other_child_cleanup, ap_null_cleanup);

    if (ap_spawn_other_child(child, func, data) == -1) {
        ap_log_error(APLOG_MARK, APLOG_ERR, s, "can't spawn other child");
        ap_kill_cleanup(p, child, ap_other_child_cleanup);
    }

#ifdef MOD_ACCEL_DEBUG
    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, s,
                 "accel: fork %d, %d", getpid(), child->pid);
#endif

}

static int ap_spawn_other_child(other_child_rec *child,
                         int (*func)(void *data), void *data)
{
    int pid;

    ap_block_alarms();
    pid = fork();
    switch (pid) {
    case -1:     /* error */
        break;

    case 0:      /* child */
        RAISE_SIGSTOP(SPAWN_CHILD);
        signal(SIGHUP, SIG_IGN);
        signal(SIGTERM, SIG_DFL);
        signal(SIGUSR1, SIG_IGN);

#ifdef MOD_ACCEL_DEBUG
    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, child->server,
                 "accel: child %d", pid);
#endif

        if ((*func)(data) == -1)
             exit(1);

        exit(0);

    default:     /* parent */
        child->pid = pid;
        ap_register_other_child(pid, ap_other_child_maintenance, child, -1);
    }
    ap_unblock_alarms();
    return pid;
}

static void ap_other_child_maintenance(int reason,
                                       void *data, ap_wait_t status)
{
    other_child_rec *child = data;

#ifdef MOD_ACCEL_DEBUG
    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_DEBUG, child->server,
                 "accel: maintenance %d, %d, %d, %04x",
                 getpid(), child->pid, reason, status);
#endif

    switch (reason) {
    case OC_REASON_DEATH:
    case OC_REASON_LOST:
        if (child->pid != -1)
            kill(child->pid, SIGKILL);
        child->pid = -1;
        ap_unregister_other_child(child);

        if (status == 0x100)
            break;

        if (ap_spawn_other_child(child, child->func, child->data) == -1) {
            ap_log_error(APLOG_MARK, APLOG_ERR, child->server,
                         "can't spawn other child");
        }

        break;

    case OC_REASON_RESTART:
        if (child->pid != -1)
            kill(child->pid, SIGKILL);
        child->pid = -1;
    }
}

static void ap_other_child_cleanup(void *data)
{
    other_child_rec *child = data;

    if (child->pid != -1)
        kill(child->pid, SIGKILL);

    ap_unregister_other_child(child);
}
